import json

import requests


def get_data_path():
    return 'data/'


def download_json_data(url, verbose=True):
    response = requests.get(url)

    if response.status_code == 200:
        data = response.json()
    else:
        data = None
        if verbose:
            print('Faulty response status code = {} for url = {}'.format(response.status_code, url))

    return data, response.status_code


def save_json_data(json_filename, json_data):
    json_as_str = json.dumps(json_data)

    with open(json_filename, 'w', encoding='utf8') as f:
        # Reference: https://stackoverflow.com/a/8710579/
        print(json_as_str, file=f)

    return True


def load_json_data(json_filename):
    with open(json_filename, 'r', encoding='utf8') as f:
        data = json.load(f)

    return data



import pathlib

import steampi.json_utils


def get_appdetails_url(app_id):
    url = 'https://store.steampowered.com/api/appdetails?appids=' + str(app_id)
    return url


def download_app_details(app_id):
    url = get_appdetails_url(app_id)
    (data, status_code) = steampi.json_utils.download_json_data(url)
    success_flag = bool(data is not None)

    downloaded_app_details = {}

    if success_flag:
        try:
            downloaded_app_details = data[app_id]['data']
        except KeyError:
            print('No data found for appID = {} with status code = {}'.format(app_id, status_code))

        try:
            success_flag = data[app_id]['success']
        except KeyError:
            success_flag = False

    return downloaded_app_details, success_flag, status_code


def get_appdetails_filename(app_id):
    # Objective: return the filename where app details for input appID are stored.

    data_path = steampi.json_utils.get_data_path() + "appdetails/"

    pathlib.Path(data_path).mkdir(parents=True, exist_ok=True)

    json_base_filename = "appID_" + str(app_id) + ".json"

    data_filename = data_path + json_base_filename

    return data_filename


def load_app_details(app_id):
    json_filename = get_appdetails_filename(app_id)

    try:
        loaded_app_details = steampi.json_utils.load_json_data(json_filename)
        success_flag = True
        status_code = None
    except FileNotFoundError:
        (loaded_app_details, success_flag, status_code) = download_app_details(app_id)
        if success_flag:
            steampi.json_utils.save_json_data(json_filename, loaded_app_details)

    return loaded_app_details, success_flag, status_code



import datetime

import steampi.api


def get_release_date_as_str(app_id):
    app_details, _, _ = steampi.api.load_app_details(app_id)

    try:
        release_date = app_details[app_id]['data']['release_date']['date']
    except KeyError:
        try:
            release_date = app_details['release_date']['date']
        except KeyError:
            release_date = None

    return release_date


def get_release_date_as_datetime(app_id):
    release_date_as_str = get_release_date_as_str(app_id)

    if release_date_as_str is not None and release_date_as_str == '':
        release_date = None
    else:

        try:
            # Reference: https://stackoverflow.com/a/6557568/
            release_date = datetime.datetime.strptime(release_date_as_str, '%d %b, %Y')
        except ValueError:
            release_date = datetime.datetime.strptime(release_date_as_str, '%b %d, %Y')
        except TypeError:
            release_date = None

    return release_date


def get_release_year(app_id):
    release_date = get_release_date_as_datetime(app_id)

    if release_date is not None:
        release_year = release_date.year
    else:
        release_year = -1

    return release_year



# noinspection PyPep8Naming
import Levenshtein as lv
import steamspypi


def compute_all_game_name_distances_with_levenshtein(input_game_name, steamspy_database=None):
    if steamspy_database is None:
        steamspy_database = steamspypi.load()

    lower_case_input = input_game_name.lower()

    text_distances = dict()

    for app_id in steamspy_database:
        text = steamspy_database[app_id]['name']

        # Compare names in lower cases, to avoid mismatches for Tekken vs. TEKKEN, or Warhammer vs. WARHAMMER
        text_distances[app_id] = lv.distance(lower_case_input, text.lower())

    return text_distances



import difflib
import heapq

import steamspypi


def compute_all_game_name_distances_with_diff_lib(input_game_name,
                                                  steamspy_database=None,
                                                  n=3,
                                                  cutoff=0.6,
                                                  ):
    lower_case_input = input_game_name.lower()

    lower_case_game_name_dictionary = build_lower_case_game_name_dictionary(steamspy_database)

    lower_case_references = lower_case_game_name_dictionary.keys()

    close_matches_and_similarity_ratios = get_close_matches_and_similarity_ratios(word=lower_case_input,
                                                                                  possibilities=lower_case_references,
                                                                                  n=n,
                                                                                  cutoff=cutoff)

    text_distances = dict()

    for (lower_case_text, similarity_ratio) in close_matches_and_similarity_ratios:
        app_id = lower_case_game_name_dictionary[lower_case_text]

        # Reference: https://docs.python.org/3/library/difflib.html#difflib.SequenceMatcher.ratio
        textual_distance = 1 - similarity_ratio

        text_distances[app_id] = textual_distance

    return text_distances


def build_lower_case_game_name_dictionary(steamspy_database=None):
    # Build a Python dictionary mapping **lower-case** game names found in SteamSpy database to their Steam appID.

    if steamspy_database is None:
        steamspy_database = steamspypi.load()

    lower_case_game_name_dictionary = dict()

    for app_id in steamspy_database:
        text = steamspy_database[app_id]['name']

        lower_case_text = text.lower()

        lower_case_game_name_dictionary[lower_case_text] = app_id

    return lower_case_game_name_dictionary


def get_close_matches_and_similarity_ratios(word, possibilities, n=3, cutoff=0.6):
    """Use SequenceMatcher to return list of the best "good enough" matches, along with the similarity ratios.

    Code inspired from:
        https://docs.python.org/3/library/difflib.html#difflib.get_close_matches

    word is a sequence for which close matches are desired (typically a
    string).

    possibilities is a list of sequences against which to match word
    (typically a list of strings).

    Optional arg n (default 3) is the maximum number of close matches to
    return.  n must be > 0.

    Optional arg cutoff (default 0.6) is a float in [0, 1].  Possibilities
    that don't score at least that similar to word are ignored.

    The best (no more than n) matches among the possibilities are returned
    in a list, sorted by similarity score, most similar first.

    >>> get_close_matches_and_similarity_ratios("appel", ["ape", "apple", "peach", "puppy"])
    [('apple', 0.8), ('ape', 0.75)]
    >>> import keyword as _keyword
    >>> get_close_matches_and_similarity_ratios("wheel", _keyword.kwlist)
    [('while', 0.6)]
    >>> get_close_matches_and_similarity_ratios("Apple", _keyword.kwlist)
    []
    >>> get_close_matches_and_similarity_ratios("accept", _keyword.kwlist)
    [('except', 0.6666666666666666)]
    """

    if not n > 0:
        raise ValueError("n must be > 0: %r" % (n,))
    if not 0.0 <= cutoff <= 1.0:
        raise ValueError("cutoff must be in [0.0, 1.0]: %r" % (cutoff,))
    result = []
    s = difflib.SequenceMatcher()
    s.set_seq2(word)
    for x in possibilities:
        s.set_seq1(x)
        if s.real_quick_ratio() >= cutoff and \
                s.quick_ratio() >= cutoff and \
                s.ratio() >= cutoff:
            result.append((s.ratio(), x))

    # Move the best scorers to head of list
    result = heapq.nlargest(n, result)
    # Swap scorers and scores for the best n matches
    close_matches_and_similarity_ratios = [(x, score) for score, x in result]

    return close_matches_and_similarity_ratios



from .difflib_utils import compute_all_game_name_distances_with_diff_lib
from .levenshtein_utils import compute_all_game_name_distances_with_levenshtein


def compute_all_game_name_distances(input_game_name,
                                    steamspy_database=None,
                                    use_levenshtein_distance=True,
                                    n=3,
                                    cutoff=0.6):
    if use_levenshtein_distance:
        # Levenshtein distance
        text_distances = compute_all_game_name_distances_with_levenshtein(input_game_name,
                                                                          steamspy_database)
    else:
        # The longest contiguous matching subsequence
        # Reference: https://docs.python.org/3/library/difflib.html
        text_distances = compute_all_game_name_distances_with_diff_lib(input_game_name,
                                                                       steamspy_database,
                                                                       n=n,
                                                                       cutoff=cutoff)

    return text_distances


def find_most_similar_game_names(input_game_name,
                                 steamspy_database=None,
                                 use_levenshtein_distance=True,
                                 n=3,
                                 cutoff=0.6):
    text_distances = compute_all_game_name_distances(input_game_name,
                                                     steamspy_database=steamspy_database,
                                                     use_levenshtein_distance=use_levenshtein_distance,
                                                     n=n,
                                                     cutoff=cutoff,
                                                     )

    sorted_app_ids = sorted(text_distances.keys(), key=lambda app_id: text_distances[app_id])

    return sorted_app_ids, text_distances



from distutils.core import setup

# noinspection PyUnresolvedReferences
import setuptools

setup(
    name='steampi',
    packages=['steampi'],
    install_requires=[
        'python-Levenshtein',
        'requests',
        'steamspypi',
    ],
    version='0.5.0',
    description='Steam API on PyPI',
    long_description='SteamPI: a simple API for Steam, written in Python 3.',
    long_description_content_type='text/x-rst',
    author='Wok',
    author_email='wok@tuta.io',
    url='https://github.com/woctezuma/steampi',
    download_url='https://github.com/woctezuma/steampi/archive/0.5.0.tar.gz',
    keywords=['steam', 'steamspy', 'api'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Games/Entertainment',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)
